/*=========================================================================
 *
 *  Copyright Insight Software Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *=========================================================================*/
#ifndef __itkQuadEdgeMeshDecimationFilter_h
#define __itkQuadEdgeMeshDecimationFilter_h

#include "itkQuadEdgeMeshToQuadEdgeMeshFilter.h"

namespace itk
{
/**
 * \class QuadEdgeMeshDecimationFilter
 * \brief
 */
template< class TInput, class TOutput, class TCriterion >
class ITK_EXPORT QuadEdgeMeshDecimationFilter:
  public QuadEdgeMeshToQuadEdgeMeshFilter< TInput, TOutput >
{
public:
  typedef QuadEdgeMeshDecimationFilter                        Self;
  typedef SmartPointer< Self >                                Pointer;
  typedef SmartPointer< const Self >                          ConstPointer;
  typedef QuadEdgeMeshToQuadEdgeMeshFilter< TInput, TOutput > Superclass;

  /** Run-time type information (and related methods).   */
  itkTypeMacro(QuadEdgeMeshDecimationFilter, QuadEdgeMeshToQuadEdgeMeshFilter);

  typedef TInput                          InputMeshType;
  typedef typename InputMeshType::Pointer InputMeshPointer;

  typedef TOutput                          OutputMeshType;
  typedef typename OutputMeshType::Pointer OutputMeshPointer;

  typedef TCriterion                                       CriterionType;
  typedef typename CriterionType::Pointer                  CriterionPointer;
  typedef typename CriterionType::MeasureType              MeasureType;
  typedef typename CriterionType::PriorityType             PriorityType;
  typedef typename CriterionType::PriorityQueueWrapperType PriorityQueueItemType;

  itkSetObjectMacro(Criterion, CriterionType);
protected:
  QuadEdgeMeshDecimationFilter()
  {
    this->m_Iteration = 0;
  }

  ~QuadEdgeMeshDecimationFilter() {}

  CriterionPointer m_Criterion;
  unsigned long    m_Iteration;

  void GenerateData()
  {
    this->CopyInputMeshToOutputMesh();

    Initialize();
    FillPriorityQueue();
    m_Iteration = 0;

    do
      {
      this->Extract();

      if ( ProcessWithTopologicalGuarantee() )
        {
        return;
        }

      ++m_Iteration;
      }
    while ( !IsCriterionSatisfied() );

    this->GetOutput()->SqueezePointsIds();
  }

  virtual void Initialize() {}
  virtual void FillPriorityQueue() = 0;

  virtual void Extract() = 0;

  virtual bool ProcessWithoutAnyTopologicalGuarantee() = 0;

  virtual bool ProcessWithTopologicalGuarantee() = 0;

  virtual bool IsCriterionSatisfied() = 0;

  void PrintSelf(std::ostream & os, Indent indent) const
  {
    this->Superclass::PrintSelf(os, indent);
    os << indent << "Criterion: " << m_Criterion << std::endl;
  }

private:
  QuadEdgeMeshDecimationFilter(const Self &);
  void operator=(const Self &);
};
}

#endif
