/*=========================================================================
 *
 *  Copyright Insight Software Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *=========================================================================*/
#ifndef __itkQuadEdgeMeshSquaredEdgeLengthDecimation_h
#define __itkQuadEdgeMeshSquaredEdgeLengthDecimation_h

#include "itkQuadEdgeMeshEdgeMergeDecimationFilter.h"

namespace itk
{
/**
 * \class QuadEdgeMeshSquaredEdgeLengthDecimation
 * \brief
 */
template< class TInput, class TOutput, class TCriterion >
class ITK_EXPORT QuadEdgeMeshSquaredEdgeLengthDecimation:
  public QuadEdgeMeshEdgeMergeDecimationFilter< TInput, TOutput, TCriterion >
{
public:
  typedef QuadEdgeMeshSquaredEdgeLengthDecimation Self;
  typedef SmartPointer< Self >                    Pointer;
  typedef SmartPointer< const Self >              ConstPointer;
  typedef QuadEdgeMeshEdgeMergeDecimationFilter<
    TInput, TOutput, TCriterion >                         Superclass;

  /** Run-time type information (and related methods).   */
  itkTypeMacro(QuadEdgeMeshSquaredEdgeLengthDecimation, QuadEdgeMeshEdgeMergeDecimationFilter);

  /** New macro for creation of through a Smart Pointer   */
  itkNewMacro(Self);

  typedef TInput                          InputMeshType;
  typedef typename InputMeshType::Pointer InputMeshPointer;

  typedef TOutput                                         OutputMeshType;
  typedef typename OutputMeshType::Pointer                OutputMeshPointer;
  typedef typename OutputMeshType::PointIdentifier        OutputPointIdentifier;
  typedef typename OutputMeshType::PointType              OutputPointType;
  typedef typename OutputMeshType::QEType                 OutputQEType;
  typedef typename OutputMeshType::EdgeCellType           OutputEdgeCellType;
  typedef typename OutputMeshType::CellsContainerIterator OutputCellsContainerIterator;

  typedef TCriterion                          CriterionType;
  typedef typename CriterionType::MeasureType MeasureType;

  typedef typename Superclass::PriorityType          PriorityType;
  typedef typename Superclass::PriorityQueueItemType PriorityQueueItemType;
  typedef typename Superclass::PriorityQueueType     PriorityQueueType;
  typedef typename Superclass::PriorityQueuePointer  PriorityQueuePointer;

  typedef typename Superclass::QueueMapType     QueueMapType;
  typedef typename Superclass::QueueMapIterator QueueMapIterator;

  typedef typename Superclass::OperatorType    OperatorType;
  typedef typename Superclass::OperatorPointer OperatorPointer;
protected:

  QuadEdgeMeshSquaredEdgeLengthDecimation();
  virtual ~QuadEdgeMeshSquaredEdgeLengthDecimation();

  /**
   * \brief Compute the measure value for iEdge
   * \param[in] iEdge
   * \return measure value, here the squared edge length
   */
  inline MeasureType MeasureEdge(OutputQEType *iEdge)
  {
    OutputMeshPointer output = this->GetOutput();

    OutputPointIdentifier id_org = iEdge->GetOrigin();
    OutputPointIdentifier id_dest = iEdge->GetDestination();

    OutputPointType org = output->GetPoint(id_org);
    OutputPointType dest = output->GetPoint(id_dest);

    return static_cast< MeasureType >( org.SquaredEuclideanDistanceTo(dest) );
  }

  /**
   * \brief
   * \param[in]
   * \return
   */
  OutputPointType Relocate(OutputQEType *iEdge);

private:
  QuadEdgeMeshSquaredEdgeLengthDecimation(const Self &);
  void operator=(const Self &);
};
}

#include "itkQuadEdgeMeshSquaredEdgeLengthDecimation.txx"
#endif
